import configparser
import logging
from app.sensing_mgr.const import Const


class ModelConfigBase:
    # DEFAULT section
    CLOUD_CONFIG = "cloud_config"
    SEND_CLOUD = "send_cloud"
    CACHE = "cache"
    SEND_INTERVAL = "send_interval"
    DATA_SEND_ONESHOT = "data_send_oneshot"
    WAIT_CONTAINER_STOP = "wait_container_stop"

    # LOG section
    LOG = "LOG"
    FILE = "file"
    STREAM = "stream"

    # ENV_sensor section
    ENV_SENSOR = "ENV_sensor"
    POLLING_INTERVAL = "polling_interval"

    MAX_INTARVAL = 3600

    def __init__(self):
        self._configs = {}
        self._logging = logging.getLogger(Const.LOGGER_NAME)

    def load(self, config_file_path):
        config = configparser.ConfigParser()
        config.read(config_file_path)

        if not self.parse_default_config(config):
            self._logging.error("config file not found [DEFAULT] section.")
            return False

        self.parse_log_config(config)

        try:
            self.parse_env_sensor_config(config)
        except Exception as e:
            self._logging.error("[ENV_sensor] section is invalid.")
            self._logging.error(e.args)

        return True

    def check_max_interval(self, val):
        try:
            if val == "":
                return 1

            if isinstance(val, str):
                val = int(val, 10)
            if val > self.MAX_INTARVAL:
                val = self.MAX_INTARVAL
                msg = print("max interval set {}[s].".format(self.MAX_INTARVAL))
                self._logging.info(msg)
            return val
        except Exception as e:
            print(e.args)
            self._logging.error("max interval check error.")
            return 1

    def parse_default_config(self, config):
        default_dict = {}
        section = "DEFAULT"
        try:
            default_dict[ModelConfigBase.CLOUD_CONFIG] = config[section].getboolean(ModelConfigBase.CLOUD_CONFIG)
            default_dict[ModelConfigBase.SEND_CLOUD] = config[section].getboolean(ModelConfigBase.SEND_CLOUD)
            default_dict[ModelConfigBase.CACHE] = config[section].getboolean(ModelConfigBase.CACHE)
            default_dict[ModelConfigBase.SEND_INTERVAL] = self.check_max_interval(config[section].getint(ModelConfigBase.SEND_INTERVAL, 30))
            default_dict[ModelConfigBase.DATA_SEND_ONESHOT] = config[section].getboolean(ModelConfigBase.DATA_SEND_ONESHOT, False)
            default_dict[ModelConfigBase.WAIT_CONTAINER_STOP] = config[section].getint(ModelConfigBase.WAIT_CONTAINER_STOP, 10)

            self._configs[section] = default_dict
            return True
        except:
            self._logging.error("[DEFAULT] section need [cloud_config], [send_cloud], [cache], [send_interval], [data_send_oneshot], [wait_container_stop]")
            return False

    def parse_log_config(self, config):
        log_dict = {}
        section = ModelConfigBase.LOG
        log_dict[ModelConfigBase.FILE] = config[section].getboolean(ModelConfigBase.FILE, True)
        log_dict[ModelConfigBase.STREAM] = config[section].getboolean(ModelConfigBase.STREAM, False)
        self._configs[section] = log_dict

    # ENV sensor
    def parse_env_sensor_config(self, config):
        env_sensor_dict = {}
        section = ModelConfigBase.ENV_SENSOR
        env_sensor_dict[ModelConfigBase.POLLING_INTERVAL] = self.check_max_interval(config[section].getint(ModelConfigBase.POLLING_INTERVAL, 1))

        self._configs[section] = env_sensor_dict

    # global
    def get_default_conf(self):
        return self._configs["DEFAULT"]

    def get_env_sensor_conf(self):
        return self._configs[ModelConfigBase.ENV_SENSOR]

    def get_log_conf(self):
        return self._configs[ModelConfigBase.LOG]

    def send_interval(self):
        conf = self.get_default_conf()
        return conf.get(ModelConfigBase.SEND_INTERVAL)

    # update
    def update_env_sensor_conf(self, key, dict):
        old_conf = None
        section = ""
        if key == ModelConfigBase.ENV_SENSOR:
            old_conf = self.get_env_sensor_conf()
            section = ModelConfigBase.ENV_SENSOR
        else:
            self._logging.error("ENV_sensor config key is invalid.")
            return None

        for key, value in dict.items():
            old_conf[key] = value

        self._configs[section] = old_conf
        return section

    def update_config(self, data):
        update_list = {}

        for key, value in data.items():
            section = None
            if "_config" not in key:
                continue

            index = key.find('_config')
            key = key[:index]

            if "ENV" in key:
                section = self.update_env_sensor_conf(key, value)

            if section:
                update_list[section] = self._configs[section]

        return update_list

#
# End of File
#
